const { some, keys, isUndefined, reduce, isArray } = require('lodash');
const minimatch = require('minimatch');
const Mock = require('mockjs');

function logger(config) {
  if (config.silent) {
    return () => {};
  } else {
    return console.log;
  }
}

/**
 *  @see: http://mockjs.com/examples.html
 */
function mockArray(array, min = 5, max = 20) {
  const key = 'array';
  return Mock.mock({
    [`${key}|${min}-${max}`]: array
  })[key];
}

function mockObject(object) {
  return Mock.mock(object);
}

function getModalDescriptor(config, pathname, method) {
  if (method) {
    config = config[method] || {};
  }
  let res;
  some(config.base, (pathnames, key) => {
    if (pathnames.indexOf(pathname) > -1) {
      res = key;
      return true;
    }
  });
  if (!res) {
    const apis = keys(config.model);
    const reg = /([^|]+)(?:\|(\d+)(?:-(\d+))?)?/;
    some(apis, api => {
      const match = api.match(reg);
      if (match && match[1] === pathname) {
        const model = config.model[api];
        if (isArray(model)) {
          res = [model];
          if (!isUndefined(match[2])) {
            res.push(match[2]);
            if (isUndefined(match[3])) {
              res.push(match[2]);
            } else {
              res.push(match[3]);
            }
          }
        } else {
          res = model;
        }

        return true;
      }
    });
  }
  return res;
}

function match(str, patterns) {
  let flag; // true: black
  return reduce(
    patterns,
    (memo, pattern) => {
      const isBlack = pattern.startsWith('!');
      if (flag === isBlack) {
        return memo;
      } else {
        flag = isBlack;
        return minimatch(str, pattern);
      }
    },
    false
  );
}

exports.match = match;
exports.logger = logger;
exports.mockArray = mockArray;
exports.mockObject = mockObject;
exports.getModalDescriptor = getModalDescriptor;
